-----------------------------------------------------------------------------
-- Name:       RecursiveCTE.sql
-- Author:     Neil Bryan
-- Date:       20130910
-- Description:The idea of this script is to demonstrate how a recurive 
--             common table expression can be used to derive hirearchial
--             data. The example given is an employee table where each record
--             pertains to an employee, which has a link to the manager record
--             in the same table. E.g.
--
--                  Employee_ID   Employee_Name   Reports_To_ID    Job_Title
--                  -----------   -------------   -------------    --------- 
--                  1             Roy Hodgson     NULL             Managing Director
--                  5095          Joe Hart        1                Accounts Director
--                  9506          Ashley Cole     5095             Strategic Account Manager
--               
-- Amendments:
-- -----------
--
-- Who                  When         Ref              Description
-- ---                  ----         ---              -----------
-- Neil Bryan           20130910     Tips003          Created
-----------------------------------------------------------------------------

--------------------------------------------------------------------------------------
-- Locals
-- @Employees - A table to store hirearchial data
-- @BossId    - The Boss's ID, from which we will locate the subservients
--------------------------------------------------------------------------------------
DECLARE @Employees TABLE (
    Employee_ID INT
    ,Employee_Name NVARCHAR(256)
    ,Reports_To_ID INT
    ,Job_Title NVARCHAR(256)
    );
DECLARE @BossId INT;

--------------------------------------------------------------------------------------
-- Set @BossID to either '5095' for all accounts workers, or '6350' for all IT workers
-- Or for everyone, set to '1' which is the ID of the managing director.
--------------------------------------------------------------------------------------
SET @BossId = 1; -- IT Director


--------------------------------------------------------------------------------------
-- Set-up test data
--------------------------------------------------------------------------------------

--
-- Managing Director
--
INSERT INTO @Employees
SELECT 1,'Roy Hodgson',NULL,'Managing Director'

--
-- Accounts Workers
--
INSERT INTO @Employees
SELECT 5095,'Joe Hart',1,'Accounts Director'
UNION
SELECT 7248,'Kyle Walker',5095,'Accounts Manager'
UNION
SELECT 9506,'AshleyCole',5095,'Strategic Account Manager'
UNION
SELECT 7250,'Gary Cahill',7248,'Service Controller'
UNION
SELECT 7251,'Phil Jagielka',7248,'Project/Design Engineer'
UNION
SELECT 7252,'Steven Gerrard',7248,'Office Administrator'
UNION
SELECT 7254,'Frank Lampard',7248,'Strategic Account Manager'
UNION
SELECT 7255,'Jack Wilshire',7254,'Area Sales Manager'
UNION
SELECT 7256,'Theo Walcott',7254,'Area Sales Manager'
UNION
SELECT 7253,'Ricky Lambert',7250,'Technical Engineer'

--
-- IT Workers
--
UNION
SELECT 6350,'Wayne Rooney',1,'IT Director'
UNION
SELECT 7200,'James Milner',6350,'Development Manager'
UNION
SELECT 8367,'Danny Welbeck',7200,'DBA';
    
--------------------------------------------------------------------------------------
-- Get the hirearchial data
--------------------------------------------------------------------------------------
WITH _hirearchy
AS (
    SELECT Employee_ID
        ,Employee_Name
        ,Reports_To_ID
        ,Job_Title
    FROM @Employees _employees
    WHERE Employee_ID = @BossId
    
    UNION ALL
    
    SELECT t.Employee_ID
        ,t.Employee_Name
        ,t.Reports_To_ID
        ,t.Job_Title
    FROM @Employees t
    INNER JOIN _hirearchy a ON t.Reports_To_ID = a.Employee_ID
    WHERE t.Employee_ID <> @BossId
    )
SELECT *
FROM _hirearchy;

